﻿using System;
using System.Net.Mail;
using System.Net.Mail.Fakes;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.QualityTools.Testing.Fakes;
using System.Fakes;

namespace Business.Test
{
    [TestClass]
    public class BankTest
    {
        [TestMethod]
        public void BankTransfer_StubExample_Interface()
        {
            /*** Arrange ***/

            // Create stub for data access
            var stub = new Data.Fakes.StubIBankRepository() 
            {
                // Return true
                DepositInt32Decimal = (account, amount) => true,
                // Return true
                WithdrawInt32Decimal = (account, amount) => true
            };

            /*** Act ***/
            // Call the business logic and pass the 
            // data access stub
            var business = new Business.Bank(stub);
            var result = business.Transfer(1, 2, 75);

            // Assert
            Assert.IsTrue(result);
        }

        [TestMethod]
        public void BankTransfer_ShimExample_Smtp()
        {
            /*** Arrange ***/

            // Create stub. (Stub won't be called in that test!)
            // We don't need to define method implemtations.
            var stub = new Data.Fakes.StubIBankRepository();

            bool result = false;
            using (ShimsContext.Create())
            {
                // "Replace" the real SmtpClient.Send implementation.
                ShimSmtpClient.Constructor = self =>
                {
                    var shim = new ShimSmtpClient(self);
                    shim.SendStringStringStringString = (from, to, subject, body) => { };
                };

                /*** Act ***/
                var business = new Business.Bank(stub);
                result = business.Transfer(1, 2, 20000);
            }

            /*** Assert ***/
            Assert.IsTrue(result);
        }

        [TestMethod]
        public void BankTransfer_ShimExample_DateTime()
        {
            /*** Arrange ***/
            var stub = new Data.Fakes.StubIBankRepository();
            var date = new DateTime(2012, 01, 03);

            DateTime result = DateTime.MinValue;
            using (ShimsContext.Create())
            {
                // Configure DateTime.Now getter.
                // Return always the same date!
                ShimDateTime.NowGet = () => date;

                /*** Act ***/
                var business = new Business.Bank(stub);
                result = business.InactiveSince(1);
            }

            /*** Assert ***/
            Assert.IsTrue(result == date);
        }
    }
}
